@@ -397,15 +397,15 @@ class PhoneNumberUtil : public Singleton<PhoneNumberUtil> {
       const string& calling_from,
       string* formatted_number) const;
 
-  // Formats a phone number using the original phone number format that the
-  // number is parsed from. The original format is embedded in the
-  // country_code_source field of the PhoneNumber object passed in. If such
-  // information is missing, the number will be formatted into the NATIONAL
-  // format by default. When we don't have a formatting pattern for the number,
-  // the method returns the raw input when it is available.
-  //
-  // Note this method guarantees no digit will be inserted, removed or modified
-  // as a result of formatting.
+  // Formats a phone number using the original phone number format (e.g.
+  // INTERNATIONAL or NATIONAL) that the number is parsed from, provided that
+  // the number has been parsed with ParseAndKeepRawInput. Otherwise the number
+  // will be formatted in NATIONAL format. The original format is embedded in
+  // the country_code_source field of the PhoneNumber object passed in, which is
+  // only set when parsing keeps the raw input. When we don't have a formatting
+  // pattern for the number, the method falls back to returning the raw input.
+  // When the number is an invalid number, the method returns the raw input when
+  // it is available.
   void FormatInOriginalFormat(const PhoneNumber& number,
                               const string& region_calling_from,
                               string* formatted_number) const;
@@ -958,8 +958,14 @@ class PhoneNumberUtil : public Singleton<PhoneNumberUtil> {
                         bool check_region,
                         PhoneNumber* phone_number) const;
 
-  void BuildNationalNumberForParsing(const string& number_to_parse,
-                                     string* national_number) const;
+  absl::optional<string> ExtractPhoneContext(
+      const string& number_to_extract_from,
+      size_t index_of_phone_context) const;
+
+  bool IsPhoneContextValid(absl::optional<string> phone_context) const;
+
+  ErrorType BuildNationalNumberForParsing(const string& number_to_parse,
+                                          string* national_number) const;
 
   bool IsShorterThanPossibleNormalNumber(const PhoneMetadata* country_metadata,
                                          const string& number) const;
